
#ifndef __ARCHIVEMANAGERPANEL_H__
#define __ARCHIVEMANAGERPANEL_H__

#include <wx/listctrl.h>
#include <wx/aui/auibook.h>
#include <wx/dirctrl.h>
#include <wx/listbox.h>
#include "ListenerAnnouncer.h"
#include "Archive.h"
#include "ListView.h"
#include "EntryPanel.h"

class ArchiveManagerPanel;

class WMFileBrowser : public wxGenericDirCtrl {
private:

public:
	ArchiveManagerPanel*	parent;

	WMFileBrowser(wxWindow* parent, ArchiveManagerPanel* wm, int id);
	~WMFileBrowser();

	void onItemActivated(wxTreeEvent &e);
};

class ArchiveManagerPanel : public wxPanel, Listener {
private:
	wxAuiNotebook*		notebook_tabs;
	wxAuiNotebook*		notebook_archives;
	ListView*			list_archives;
	ListView*			list_recent;
	ListView*			list_bookmarks;
	wxListCtrl*			list_maps;
	WMFileBrowser*		file_browser;
	wxButton*			btn_browser_open;
	wxMenu*				menu_context_open;
	wxMenu*				menu_context_recent;
	wxMenu*				menu_context_bookmarks;
	wxMenu*				menu_recent;

public:
	// wxWidgets ID table
	enum {
		// Open Achives menu items
		MENU_SAVE,
		MENU_SAVEAS,
		MENU_CLOSE,

		// Recent Files menu items
		MENU_OPEN,
		MENU_REMOVE,

		// Bookmarks menu items
		MENU_GO,
		MENU_DELETE,

		// End
		MENU_END,
	};

	ArchiveManagerPanel(wxWindow *parent, wxAuiNotebook* nb_archives);
	~ArchiveManagerPanel();

	wxMenu*			recentFilesMenu() { return menu_recent; }

	void			refreshArchiveList();
	void			refreshRecentFileList();
	void			refreshBookmarkList();
	void			refreshAllTabs();
	void			populateMapList(Archive* archive);
	void			updateOpenListItem(int index);
	void			updateRecentListItem(int index);
	void			updateBookmarkListItem(int index);
	bool			isArchivePanel(int tab_index);
	Archive*		getArchive(int tab_index);
	int				currentTabIndex();
	Archive*		currentArchive();
	wxWindow*		currentPanel();
	EntryPanel*		currentArea();

	ArchiveEntry*			currentEntry();
	vector<ArchiveEntry*>	currentEntrySelection();

	void	openTab(int archive_index);
	void	openTab(Archive * archive);
	void	closeTab(int archive_index);
	void	openTextureTab(int archive_index);
	void	closeTextureTab(int archive_index);
	void	openFile(string filename);
	void	openFiles(wxArrayString& files);

	// Single archive actions
	bool	saveEntryChanges(Archive* archive);
	bool	saveArchive(Archive* archive);
	bool	saveArchiveAs(Archive* archive);
	bool	closeArchive(Archive* archive);

	void	createNewArchive(uint8_t type);
	bool	closeAll();
	void	saveAll();

	// Selected archives in the lists
	void	saveSelection();
	void	saveSelectionAs();
	bool	closeSelection();
	void	openSelection();
	void	removeSelection();

	// Bookmark functions
	void	deleteSelectedBookmarks();
	void	goToBookmark(long index = -1);

	// Handler for MainWindow menu actions
	void	handleAction(int menu_id);

	vector<int>	getSelectedArchives();
	vector<int>	getSelectedBookmarks();
	vector<int>	getSelectedFiles();

	void	onAnnouncement(Announcer* announcer, string event_name, MemChunk& event_data);

	// Event handlers
	void	onListArchivesChanged(wxListEvent& e);
	void	onListArchivesActivated(wxListEvent& e);
	void	onListArchivesRightClick(wxListEvent& e);
	void	onListRecentChanged(wxListEvent& e);
	void	onListRecentActivated(wxListEvent& e);
	void	onListRecentRightClick(wxListEvent& e);
	void	onListBookmarksActivated(wxListEvent& e);
	void	onListBookmarksRightClick(wxListEvent& e);
	void	onListMapsChanged(wxCommandEvent& e);
	void	onListMapsActivated(wxListEvent& e);
	void	onMenu(wxCommandEvent& e);
	void	onArchiveTabChanged(wxAuiNotebookEvent& e);
	void	onArchiveTabClose(wxAuiNotebookEvent& e);
	void	onAMTabChanged(wxAuiNotebookEvent& e);
};

#endif //__ARCHIVEMANAGERPANEL_H__
